/*
 * \file
 * X-Forge Engine <br>
 * Copyright 2000-2003 Fathammer Ltd
 *
 * \brief Remote debug server
 *
 * $Id: XFuRemoteDebug.h,v 1.6 2003/05/23 09:54:04 slehti Exp $
 * $Date: 2003/05/23 09:54:04 $
 * $Revision: 1.6 $
 */

#ifndef XFUREMOTEDEBUG_H_INCLUDED
#define XFUREMOTEDEBUG_H_INCLUDED

#include <xfcore/net/XFcRemoteCommunicationRecv.h>
#include <xfcore/XFcHashtable.h>

static const INT32 RECV_BUFFER_MAX = 128;
static const INT32 RECEIVER_NAME_MAX = 10; 
static const CHAR8 RECEIVE_REPASS_BUFFER_CHAR = 0x0d;

class XFuRemoteDebugService;
class XFcRemoteCommunication;

class XFuRemoteDebug : public XFcRemoteCommunicationRecv
{
private:

    //! Holds service hash table.
    XFcHashtable <XFuStringKey, XFuRemoteDebugService *> mService;

    //! Holds buffer for receiving data, temporary!
    CHAR8 mRecvBuffer[128];

    //! Holds buffer position.
    INT32 mPosition;

    //! Holds pointer to remote connection server
    XFcRemoteCommunication *mRemoteComm;
    
protected:

    /*! Constructor, empty implementation.
     */
    XFuRemoteDebug();

    /*!
     * Initializes the Remote Debugger and registers it to XForge core.
     * \return 1 if success else 0.
     */
    INT init(INT32 aPort);

    /*! Callback for receiving data.
     * \param aData holds buffer of incoming data.
     * \param aSize holds size of the buffer.
     * \param aError holds receive error.
     */
    virtual void receive(const CHAR8 * aData, INT32 aSize, INT32 aError);

    /*! Event callback from remote debug server.
     * \param aError is event status.
     */
    virtual void event(INT32 aError);

public:

    /*! static create
     */
    static XFuRemoteDebug * create(INT32 aPort);

    /*! Sends data to remote debugger.
     * \param aData pointer to string.
     * \return sent byte count.
     */
    INT32 send(const CHAR8 *aData);

    /*! Sends data to remote debugger.
     * \param aData pointer to buffer.
     * \param aSize size of the data buffer.
     * \return sent byte count.
     */
    INT32 send(const CHAR8 *aData, INT32 aSize);

    /*! Register remote debug service.
     * \param aService remote debug service.
     * \return 1 if register wass successfully else 0.
     */
    INT registerService(const CHAR *aName, XFuRemoteDebugService &aService);

    /*! Removes registered service.
     * \param aName of the service.
     * \return pointer to service.
     */
    XFuRemoteDebugService * uregisterService(const CHAR *aName);



    /*! Destructor, removes remote debugger from core.
     */
    virtual ~XFuRemoteDebug();


};

#endif
