/*! \file 
 * X-Forge Util <br>
 * Copyright 2000-2003 Fathammer Ltd
 * 
 * \brief RIFF Wave loader
 * 
 * $Id: XFuWavLoad.cpp,v 1.5 2003/03/20 13:21:09 jetro Exp $
 * $Date: 2003/03/20 13:21:09 $
 * $Revision: 1.5 $
 */

#include <xfcore/XFcCore.h>
#include <xfutil/XFuWavLoad.h>


class TempData
{
public:
    XFcFile *mF;
    INT16 mWavFormat;
    INT16 mChannels;
    INT32 mSampleRate;
    INT16 mBytesPerSample;
    INT32 mTotalBytes;
    void *mData;
};


// All the formats we're even remotely likely to support here:
//#define WAVE_FORMAT_PCM         0x0001
//#define WAVE_FORMAT_ADPCM       0x0002  /*  Microsoft Corporation  */
//#define WAVE_FORMAT_IEEE_FLOAT  0x0003  /*  Microsoft Corporation  */
//                                        /*  IEEE754: range (+1, -1]  */
//                                        /*  32-bit/64-bit format as defined by */
//                                        /*  MSVC++ float/double type */
//#define  WAVE_FORMAT_ALAW       0x0006  /*  Microsoft Corporation  */
//#define  WAVE_FORMAT_MULAW      0x0007  /*  Microsoft Corporation  */
//#define  WAVE_FORMAT_MPEGLAYER3 0x0055  /*  ISO/MPEG Layer3 Format Tag */

// currently this loader only supports 8bit mono PCM data.


static INT loadHeader(TempData &d)
{
    INT32 t; 
    INT32 chunkSize;
    
    d.mF->read(&t, 4, 1);
    if (t != 0x46464952) return 0; // "RIFF" reversed
    d.mF->read(&t, 4, 1); // skip file len
    d.mF->read(&t, 4, 1);
    if (t != 0x45564157) return 0; // "WAVE" reversed
    d.mF->read(&t, 4, 1);
    if (t != 0x20746d66) return 0; // "fmt " reversed
    d.mF->read(&chunkSize, 4, 1);
    d.mF->read(&d.mWavFormat, 2, 1);
    if (!(d.mWavFormat == 0x0001)) return 0; // unsupported format
    d.mF->read(&d.mChannels, 2, 1);
    if (!(d.mChannels == 1)) return 0; // unsupported number of mChannels
    d.mF->read(&d.mSampleRate, 4, 1);
    d.mF->read(&t, 4, 1); // skip "avgbytespersec"
    d.mF->read(&d.mBytesPerSample, 2, 1);
    if (!(d.mBytesPerSample == 1)) return 0; // unsupported bit/chn combination
    chunkSize -= 2 + 2 + 4 + 4 + 2;
    
    // skip the rest of the header, if any: 
    // (microsoft PCM has 2 more bytes, bits per sample,
    // which we can figure out already)
    d.mF->seek(d.mF->tell() + chunkSize, SEEK_SET);
    
    return 1;
}


XFcAudioBuffer * xfuLoadWav(const CHAR *filename, INT32 flags)
{
    TempData d;
    
    d.mData = NULL;
    d.mF = XFcFile::open(filename, XFCSTR("rb"));
    if (d.mF == NULL) return NULL;
    if (!loadHeader(d)) 
    {
        d.mF->close();
        return NULL;
    }
    INT32 chunkId;
    
    // while not end of file
    while (d.mF->read(&chunkId, 4, 1)) 
    {
        INT32 chunkSize;
        d.mF->read(&chunkSize, 4, 1);
        if (chunkId == 0x61746164 && d.mData == NULL)
        {
            d.mData = (void *)new INT8[chunkSize];
            if (d.mData == NULL)
            {
                d.mF->close();
                return NULL;
            }
            d.mF->read(d.mData, chunkSize, 1);
            d.mTotalBytes = chunkSize;
        }
        else
        {
            // skip chunk:
            d.mF->seek(d.mF->tell() + chunkSize, SEEK_SET);
        }
    }
    d.mF->close();
    
    XFcAudioBuffer *sndBuf;
    // FIX ME: not d.mTotalBytes but TOTAL SAMPLES
    sndBuf = XFcAudioBuffer::create((FLOAT32)d.mSampleRate, flags & ~LW_SCALEHALF, d.mTotalBytes, 1.0, 0.5);

    if (sndBuf == NULL)
    {
        delete[] (INT8 *)d.mData;
        return NULL;
    }

    INT8 *buf = (INT8 *)XFcAudio::lock(sndBuf);
    if (flags & LW_SCALEHALF)
    {
        INT32 i;
        for (i = 0; i < d.mTotalBytes; ++i)
        {
            buf[i] = (INT8)(((((INT8 *)d.mData)[i] ^ 128) / 8) ^ 128);
        }
    }
    else
    {
        memcpy(buf, d.mData, d.mTotalBytes);
    }

    XFcAudio::unlock(sndBuf);

    delete[] (INT8 *)d.mData;

    return sndBuf;
}


XFcAudioBuffer * xfuLoadWav(const CHAR *filename)
{
    return xfuLoadWav(filename, 0);
}
